jQuery.sap.require('sas.risk.common.utils.SASStudioProxy');

jQuery.sap.declare('sas.risk.irm.pfdcommon.UI5DiagramBridge');

sas.risk.irm.pfdcommon.UI5DiagramBridge = function () {
    "use strict";
    var UTILS = sas.risk.irm.SIIUtils,
        CONSTANTS = sas.risk.irm.SIIConstants,
        uploadDialog = null;

    /**
     * Determines if a context menu button should be visible.  The button is invisible if
     * the status is unavailable, or if an id was supplied and that id is not in the list of
     * initial input nodes for the flow
     * @param {number} status The status of the node
     * @param {string} id The id of the node in GoJS terms
     * @param {Array} initialInputNodeIds The list of ids of the nodes that are initial inputs to the flow
     * @returns {boolean}
     */
    function isContextMenuRowVisible(status, id, initialInputNodeIds) {
        if (status === SIIConstants.STATUS_UNAVAILABLE) {
            return false;
        }
        else if (id) {
            return (initialInputNodeIds.indexOf(id) > -1)
        }
        return true;
    }


    /**
     * Is the open in SAS Studio menu option enabled
     * @param {Object} nodeData The data for the node as held in the diagram
     * @param {Array} initialInputNodeIds The list of ids of the nodes that are initial inputs to the flow
     * @returns {boolean}
     */
    function isOpenInSASStudioMenuItemEnabled(nodeData) {
        return nodeData.modelObject.isOpenInSASStudioAllowed();
    }


    /**
     * Is the export menu option enabled
     * @param {Object} nodeData The data for the node as held in the diagram
     * @param {Array} initialInputNodeIds The list of ids of the nodes that are initial inputs to the flow
     * @returns {boolean}
     */
    function isExportMenuItemEnabled(nodeData) {
        return nodeData.modelObject.isDownloadAllowed()
    }

    /**
     * Is the import menu option enabled
     * @param {Object} nodeData The data for the node as held in the diagram
     * @returns {boolean}
     */
    function isImportMenuItemEnabled(nodeData) {
        // if native data type is xls or xbrl or if the node is not editable then we can't import
        return nodeData.modelObject.isUploadAllowed();
    }

    /**
     * Is the open external url menu option enabled
     * @param {Object} nodeData The data for the node as held in the diagram
     * @returns {boolean}
     */
    function isUrlMenuItemEnabled(nodeData) {
        return nodeData.modelObject.hasUrl();
    }

    function isVersionsMenuItemEnabled(nodeData) {
        return nodeData.modelObject.hasVersions();
    }

    /**
     * Returns the list of menu items for the passed in data node nodeData object.  This will return at least 1 item.  The first
     * item in the list should be used as the header row and styled appropriately.
     * @param {?string} dataNodeKey The key for the data node
     * @param {number} instanceKey The key for the instance of the flow
     * @param {Object} nodeData The data for the node as held in the diagram
     * @param {boolean} nodeEditable Is the node editable
     * @param {Object} metadata The metadata that was downloaded as a part of the instance information for the node
     * @param {Array} initialInputNodeIds The list of ids of the nodes that are initial inputs to the flow
     * @param {Object} owningController The controller that will own the context menu
     * @returns {Array}
     */
    function getDataNodeMenuItems(dataNodeKey, instanceKey, nodeData, nodeEditable, metadata, initialInputNodeIds, owningController) {
        var menuItems = [];

        var nodeName = metadata.name;

        var titleMenuItem = new sas.hc.ui.unified.MenuItem({id: 'dataNodeTitleMenuItem', text:nodeName});
        titleMenuItem.setEnabled(false);
        menuItems.push(titleMenuItem);

        var rb = sap.ui.getCore().getLibraryResourceBundle(CONSTANTS.RB_MESSAGE_BUNDLE);
        if (SIIConstants.SHOW_STUDIO && isOpenInSASStudioMenuItemEnabled(nodeData)) {
            var openInSASStudioItem = new sas.hc.ui.unified.MenuItem({
                id: 'dataNodeOpenInSASStudioMenuItem',
                text: rb.getText(CONSTANTS.RB_OPEN_IN_SAS_STUDIO)
            });
            openInSASStudioItem.attachSelect(function () {
                var promise = SIIUtils.ajax({
                    headers: {  // S1363447 handle DBCS in webdav url
                        Accept : "text/plain; charset=utf-8",
                        "Content-Type": "text/plain; charset=utf-8"
                    },
                    url: SIIUtils.getURL(SIIConstants.URL_JOBFLOW_INSTANCES_WEBDAVEURL_DATA + "/" + instanceKey + "/webdavlibname/data/" + dataNodeKey),
                    type: "GET",
                    dataType: "text"
                });
                promise.fail(function (data) {
                    SIIUtils.logAjaxError("Failed to retrieve webdav libname for instance: " + instanceKey + " data node: " + dataNodeKey, promise);
                });
                promise.done(function (libnameStmt) {
                    UTILS.log("SAS Studio GSUBMIT: " + libnameStmt);
                    setTimeout(function () {
                        var fileNameNoExtension = metadata.name.replace(/\.sas7bdat$/,"");
                        sas.risk.common.utils.SASStudioProxy.launchStudio();
                        sas.risk.common.utils.SASStudioProxy.submitCode(libnameStmt);
                        sas.risk.common.utils.SASStudioProxy.openTable(fileNameNoExtension);
                    }, 1);
                });

            });
            menuItems.push(openInSASStudioItem);
        }

        if (isExportMenuItemEnabled(nodeData)) {
            var exportMenuItem = new sas.hc.ui.unified.MenuItem({
                id: 'dataNodeExportMenuItem',
                text: rb.getText(CONSTANTS.RB_DIAGRAM_DOWNLOAD)
            });
            exportMenuItem.attachSelect(function () {
                openPfdDataNodeURLInternal(dataNodeKey, instanceKey, metadata);
            });
            menuItems.push(exportMenuItem);
        }

        if (isImportMenuItemEnabled(nodeData)) {
            var importMenuItem = new sas.hc.ui.unified.MenuItem({
                id: 'dataNodeImportMenuItem',
                text: rb.getText(CONSTANTS.RB_DIAGRAM_UPLOAD)
            });
            importMenuItem.attachSelect(function () {
                if (metadata.status === CONSTANTS.STATUS_UNAVAILABLE) {
                    showNodeNotAvailableInternal();
                }
                else {
                    importSASDatasetInternal(nodeData, metadata, instanceKey, dataNodeKey, owningController);
                }
            });
            menuItems.push(importMenuItem);
        }

        if (isUrlMenuItemEnabled(nodeData)) {
            var urlMenuItem = new sas.hc.ui.unified.MenuItem({
                id: 'dataNodeUrlMenuItem',
                text: rb.getText(CONSTANTS.RB_DIAGRAM_OPEN_URL)
            });
            urlMenuItem.setTooltip(nodeData.modelObject.getExtUrlTooltip());
            urlMenuItem.attachSelect(function () {
                window.open(nodeData.modelObject.getExtUrl());
            });
            menuItems.push(urlMenuItem);
        }

        if(isVersionsMenuItemEnabled(nodeData)) {
            var versionsMenuItem = new sas.hc.ui.unified.MenuItem({
                id: 'dataNodeVersionsMenuItem',
                text: rb.getText(CONSTANTS.RB_VIEW_VERSIONS)
            });
            versionsMenuItem.attachSelect(function (evt) {
                owningController.openVersionsDialog(nodeData.modelObject);
            });
            menuItems.push(versionsMenuItem);
        }

        return menuItems;
    }

    /**
     * Returns true if the url supplied is a relative URL, meaning it doesn't start with http
     * @param url
     */
    function isRelativeUrl(url) {
        return ! (/^HTTP/).test(url.toUpperCase())
    }

    /**
     * Returns the list of menu items for the passed in data node nodeData object.  This will return at least 1 item.  The first
     * item in the list should be used as the header row and styled appropriately.
     * @param {Object} nodeData - The data for the node as held in the diagram
     * @param {Object} metadata - The metadata that was downloaded as a part of the instance information for the node
     * @returns {Array}
     */
    function getTaskNodeMenuItems(nodeData, metadata) {
        var menuItems = [];

        var nodeName = nodeData.displayName;

        // add the menu title
        var titleMenuItem = new sas.hc.ui.unified.MenuItem({id: 'dataNodeTitleMenuItem', text:nodeName});
        titleMenuItem.setEnabled(false);
        menuItems.push(titleMenuItem);

        var rb = sap.ui.getCore().getLibraryResourceBundle(CONSTANTS.RB_MESSAGE_BUNDLE);
        // Add the log menu item
        if (nodeData.type === "serviceTask") {
            var status = nodeData.status;
            if (status === CONSTANTS.STATUS_COMPLETED_ERROR || status === CONSTANTS.STATUS_COMPLETED_SUCCESS) {
                var showLogMenuItem = new sas.hc.ui.unified.MenuItem({
                    id: 'taskShowLogMenuItem',
                    text: rb.getText(CONSTANTS.RB_DIAGRAM_SHOW_LOG)
                });
                showLogMenuItem.attachSelect(function () {
                    var status = nodeData.status;
                    if (status !== CONSTANTS.STATUS_COMPLETED_ERROR && status !== CONSTANTS.STATUS_COMPLETED_SUCCESS) {
                        return;
                    }

                    var bus = sap.ui.getCore().getEventBus();
                    if (nodeData.name) { // start and end nodes won't have a name
                        bus.publish(CONSTANTS.EVENTBUS_CHANNEL_ACTION, CONSTANTS.EVENT_OPEN_PFD_TASK_NODE_URL, { nodeData: nodeData});
                    }
                });
                menuItems.push(showLogMenuItem);

                var downloadLogMenuItem = new sas.hc.ui.unified.MenuItem({
                    id: 'taskDownloadLogMenuItem',
                    text: rb.getText(CONSTANTS.RB_DIAGRAM_DOWNLOAD_LOG)
                });
                downloadLogMenuItem.attachSelect(function () {
                    var status = nodeData.status;
                    if (status !== CONSTANTS.STATUS_COMPLETED_ERROR && status !== CONSTANTS.STATUS_COMPLETED_SUCCESS) {
                        return;
                    }

                    var nodeMetadata = {
                        name: metadata.name,
                        displayName: SIIUtils.getDisplayName(metadata.name, true, SIIUtils.getJobFlowMessages()),
                        uri: UTILS.getURL(metadata.logFileUrl)
                    };

                    var bus = sap.ui.getCore().getEventBus();
                    if (nodeData.name) { // start and end nodes won't have a name
                        bus.publish(CONSTANTS.EVENTBUS_CHANNEL_ACTION, CONSTANTS.EVENT_DOWNLOAD_FILE, nodeMetadata);
                    }
                    menu.close();
                });
                menuItems.push(downloadLogMenuItem);

            }
        }
        else {
            // is a sub process so it must be openable
            var openMenuItem = new sas.hc.ui.unified.MenuItem({
                id: 'taskNodeOpenMenuItem',
                text: rb.getText(CONSTANTS.RB_DIAGRAM_OPEN_SUBFLOW)
            });
            openMenuItem.attachSelect(function () {
                var bus = sap.ui.getCore().getEventBus();
                bus.publish(CONSTANTS.EVENTBUS_CHANNEL_ACTION, CONSTANTS.EVENT_OPEN_SUBFLOW, { nodeData:nodeData });
                menu.close();
            });
            menuItems.push(openMenuItem);
        }

        // Add the Help menu item
        if (nodeData.helpLinks && nodeData.helpLinks.helpDocumentLink) {
            var showHelpMenuItem = new sas.hc.ui.unified.MenuItem({
                id: 'taskShowHelpMenuItem',
                text: rb.getText(CONSTANTS.RB_DIAGRAM_SHOW_HELP)
            });
            showHelpMenuItem.attachSelect(function () {
                var documentLink = nodeData.helpLinks.helpDocumentLink;
                if (isRelativeUrl(documentLink) && metadata.baseDocUrl) {
                    documentLink = CONSTANTS.BASE_URL + metadata.baseDocUrl + "/" + nodeData.helpLinks.helpDocumentLink;
                }
                var bus = sap.ui.getCore().getEventBus();
                bus.publish(CONSTANTS.EVENTBUS_CHANNEL_ACTION, CONSTANTS.EVENT_OPEN_HELP_DOCUMENT, { url: documentLink });
                menu.close();
            });
            menuItems.push(showHelpMenuItem);

        }
        return menuItems;
    }

    /**
     * Builds the view that shows user's comments for why a modification was made
     * @param {Object} modifiedInfo holds specific information about the modification
     * @returns {sas.hc.ui.layout.form.Form}
     */
    function buildCommentsView(modifiedInfo) {
        var modifiedByField = new sas.hc.ui.commons.Label( {
            text: modifiedInfo.createdBy
        });
        modifiedByField.addStyleClass("IRMPropertyValue");

        var modifiedTimeField = new sas.hc.ui.commons.Label({
            text: SIIUtils.dateTimeFormatter(modifiedInfo.createTimestamp)
        });
        modifiedTimeField.addStyleClass("IRMPropertyValue");

        var commentsField = new sas.hc.m.Text({
            text: modifiedInfo.notes,
            editable: false
        });
        commentsField.addStyleClass("IRMPropertyValue");

        var rb = sap.ui.getCore().getLibraryResourceBundle(SIIConstants.RB_MESSAGE_BUNDLE);
        var commentsForm = new sas.hc.ui.layout.form.Form({
            layout: new sas.hc.ui.layout.form.ResponsiveGridLayout({
                columnsL: 1
            }),
            formContainers: [
                new sas.hc.ui.layout.form.FormContainer({
                    formElements: [
                        new sas.hc.ui.layout.form.FormElement( {
                            label: rb.getText(CONSTANTS.RB_MODIFIED_BY_LABEL),
                            fields: [
                                modifiedByField
                            ]
                        }),
                        new sas.hc.ui.layout.form.FormElement({
                            label: rb.getText(CONSTANTS.RB_MODIFIED_TIME_LABEL),
                            fields: [
                                modifiedTimeField
                            ]
                        }),
                        new sas.hc.ui.layout.form.FormElement({
                            label: rb.getText(CONSTANTS.RB_MODIFIED_NOTES_LABEL),
                            fields: [
                                commentsField
                            ]
                        })
                    ]
                })
            ]
        });
        return commentsForm;
    }

    /**
     * Builds the view that shows the FA the data object lives in.
     * @param {String} faName holds the name of the FA the data object is in
     * @returns {sas.hc.ui.layout.form.Form}
     */
    function buildFANameView(faName) {
        var rb = sap.ui.getCore().getLibraryResourceBundle(SIIConstants.RB_MESSAGE_BUNDLE);

        var faNameField = new sas.hc.ui.commons.Label( {
            text: faName
        });
        faNameField.addStyleClass("IRMPropertyValue");

        var faNameForm = new sas.hc.ui.layout.form.Form({
            layout: new sas.hc.ui.layout.form.ResponsiveGridLayout({
                columnsL: 1
            }),
            formContainers: [
                new sas.hc.ui.layout.form.FormContainer({
                    formElements: [
                        new sas.hc.ui.layout.form.FormElement( {
                            label: rb.getText(CONSTANTS.RB_FA_NAME_POPOVER_LABEL),
                            fields: [
                                faNameField
                            ]
                        })
                    ]
                })
            ]
        });
        return faNameForm;
    }

    /**
     * Executes the action of importing a SAS Dataset by opening the appropriate dialog
     * @param {Object} nodeData The data for the node as held in the diagram
     * @param {Object} metadata The metadata that was downloaded as a part of the instance information for the node
     * @param {string} instanceKey The key for the instance
     * @param {string} dataKey The key for the data node
     * @param {Object} owningController The controller that will own the context menu
     */
    function importSASDatasetInternal(nodeData, metadata, instanceKey, dataKey, owningController) {
        var rb = sap.ui.getCore().getLibraryResourceBundle(SIIConstants.RB_MESSAGE_BUNDLE);

        // If the flow is running don't allow them to open up a file
        UTILS.log("importSASDataset called");

        var id = nodeData.id;

        var dataNodeMetadata = {
            name: metadata.name,
            url: metadata.url,
            instanceKey: instanceKey, // getInstanceOrSubflowKey(),
            dataKey: dataKey,
            nodeId: id,
            isGeneric: ((metadata.hasOwnProperty("casLibValid") && metadata.casLibValid === true) ||
                        (metadata.hasOwnProperty("librefValid") && metadata.librefValid === true))
        };
        var uploadModel = new sap.ui.model.json.JSONModel();
        uploadModel.setData(dataNodeMetadata);

        if (uploadDialog) {
            uploadDialog.destroy();
        }

        uploadDialog = sap.ui.jsfragment("sas.risk.irm.FileUpload", owningController);
        uploadDialog.setTitle(rb.getText(SIIConstants.RB_FILEUPLOAD_TITLE));
        uploadDialog.setModel(uploadModel);
        uploadDialog.open();
    }

    /**
     * Takes the action of opening the data of a data node (this usually means downloading the file)
     * @param {string} dataNodeKey The key for the data node
     * @param {string} instanceKey The key for the instance
     * @param {Object} metadata The metadata that was downloaded as a part of the instance information for the node
     */
    function openPfdDataNodeURLInternal(dataNodeKey, instanceKey, metadata) {
        // If the flow is running don't allow them to open up a file
        UTILS.log("Opening pfd data Node URL called");

        var dataNodeMetadata = {
            name: metadata.name,
            url: metadata.url,
            instanceKey: instanceKey,
            dataKey: dataNodeKey,
            isGeneric: ((metadata.hasOwnProperty("casLibValid") && metadata.casLibValid === true) ||
                        (metadata.hasOwnProperty("librefValid") && metadata.librefValid === true))
        };

        // open the file
        UTILS.openFile(dataNodeMetadata, false);
    }

    /**
     * Shows the node not available dialog
     */
    function showNodeNotAvailableInternal() {
        UTILS.showErrorDialog(SIIConstants.RB_ERROR_NODENOTAVAILABLE,SIIConstants.RB_ERROR_NODENOTAVAILABLE_TITLE);
    }

    return {

        /**
         * Takes the action of opening the data of a data node (this usually means downloading the file)
         * @param {string} dataNodeKey The key for the data node
         * @param {string} instanceKey The key for the instance
         * @param {Object} metadata The metadata that was downloaded as a part of the instance information for the node
         */
        openPfdDataNodeURL: function(dataNodeKey, instanceKey, metadata) {
            openPfdDataNodeURLInternal(dataNodeKey, instanceKey, metadata);
        },

        /**
         * Shows the log file for a task node from the diagram.
         * If the flow is running or has an unknown status an appropriate message will
         * be shown instead.
         * @param {number} flowStatus The current status of the flow
         * @param {string} taskDisplayName The display name for the task
         * @param {string} taskKey The key for the task
         * @param {string} instanceKey The key for the instance
         * @param {Object} metadata The metadata that was downloaded as a part of the instance information for the node
         */
        openPfdTaskNodeURL: function(flowStatus, taskDisplayName, taskKey, instanceKey, metadata) {
            // If the flow is running don't allow them to open up a file
            UTILS.log("Opening pfd task Node URL called");

            if (flowStatus === SIIConstants.STATUS_UNKNOWN) {
                UTILS.showErrorDialog(SIIConstants.RB_ERROR_OPENLOGBEFORERUN,SIIConstants.RB_ERROR_OPENLOGBEFORERUN_TITLE);
                return;
            }

            var oFileData = {
                url: metadata.logFileUrl,
                name: metadata.name,
                displayName: taskDisplayName,
                taskKey:taskKey,
                instanceKey: instanceKey
            };

            // open the file
            UTILS.openFile(oFileData, true);
        },

        /**
         * Opens the help document for the given nodeData
         * @param {Object} nodeData The data for the node as held in the diagram
         */
        openHelpDocument: function(nodeData) {
            if (nodeData.url) {
                window.open(nodeData.url);
            }
            else {
                UTILS.logError("URL missing for help data");
            }
        },

        /**
         *
         * @param {string} dataNodeKey The key for the data node
         * @param {string} instanceKey The key for the instance
         * @param {Object} oData The goJS diagram data object
         * @param {boolean} nodeEditable Is the node editable
         * @param {Object} metadata The metadata that was downloaded as a part of the instance information for the node
         * @param {Array} initialInputNodeIds The list of ids of the nodes that are initial inputs to the flow
         * @param owningController
         */
        showNodeContextMenu: function(dataNodeKey, instanceKey, oData, nodeEditable, metadata, initialInputNodeIds, owningController) {

            // Get the overlay div and size it and move it to cover the node that we will show the context menu for
            var contextDiv = $('#context_div');
            var panel = $('#WorkflowDiagram');
            var offsetX = panel.offset().left;
            var offsetY = panel.offset().top;

            var width = oData.viewLoc.width;
            var height = oData.viewLoc.height;

            var x = oData.viewLoc.x;
            var y = oData.viewLoc.y-height;

            var menu = sap.ui.getCore().byId('dataNodeContextMenu');
            contextDiv.css('left',x);
            contextDiv.css('top',y);
            contextDiv.css("width", width);
            contextDiv.css("height", height);

            // Uncomment to visually see where the overlay div was moved to
            // overlayDiv.css('background-color','green');
            contextDiv.show();

            // Build the menu items for the node
            var nodeData = oData.nodeData;
            menu.destroyItems();
            if (!metadata) {
                // If we got no metadata - ABORT, ABORT!!
                return;
            }

            var menuItems;
            if (nodeData.type === "subProcess" || nodeData.type === "serviceTask") {
                menuItems = getTaskNodeMenuItems(nodeData, metadata);
            }
            else {
                menuItems = getDataNodeMenuItems(dataNodeKey, instanceKey, nodeData, nodeEditable, metadata, initialInputNodeIds, owningController);
            }

            // No need to show the menu if no items would be selectable
            if (menuItems.length === 1) {
                //Have to destroy the title menu item to avoid a duplicate id error
                menuItems[0].destroy(); // Could just not give it an id, if we determine it doesn't need styling
                return;
            }
            // Add the menu items to the context menu
            for (var i=0; i< menuItems.length; i++) {
                menu.addItem(menuItems[i]);
            }

            // Open the menu
            var eDock = sap.ui.core.Popup.Dock;
            menu.open(
                false /*First item already highlighted*/,
                null /*Dom reference which gets the focus back when the menu is closed*/,
                eDock.BeginTop, /*"Edge" of the menu (see sap.ui.core.Popup)*/
                eDock.BeginBottom, /*"Edge" of the related opener position (see sap.ui.core.Popup)*/
                contextDiv); /*Related opener position (see sap.ui.core.Popup)*/
        },

        /**
         * Action to import a SASDataset into the diagram.
         * If the flow is running or has an unknown status an appropriate message will
         * be shown instead.
         * @param {Object} nodeData The data for the node as held in the diagram
         * @param {Object} metadata The metadata that was downloaded as a part of the instance information for the node
         * @param {string} instanceKey The key for the instance
         * @param {string} dataKey The key for the data node
         * @param {Object} owningController The controller that will own the context menu
         */
        importSASDataset: function(nodeData, metadata, instanceKey, dataKey, owningController) {
            importSASDatasetInternal(nodeData, metadata, instanceKey, dataKey, owningController);
        },

        /**
         * Shows the node not available dialog
         */
        showNodeNotAvailable: function() {
            showNodeNotAvailableInternal();
        },

        /**
         * Called to clean up the dialogs owned by the Diagram Bridge.
         */
        cleanupDialogs: function() {

            if(uploadDialog) uploadDialog.destroy();

        }
    }
};